package com.nokia.m2m.sdk.examples.device;

import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;

import org.omg.CORBA.Any;
import org.omg.CORBA.ORB;
import org.omg.CORBA.Object;

import com.nokia.m2m.orb.idl.properties.ParametersPackage.HLAInformation;
import com.nokia.m2m.orb.idl.properties.ParametersPackage.HLAInformationHelper;
import com.nokia.m2m.orb.idl.properties.ParametersPackage.PPPNegotiationIPAddressHelper;
import com.nokia.m2m.orb.idl.properties.ParametersPackage.WAPBearerInformation;
import com.nokia.m2m.orb.idl.properties.ParametersPackage.WAPBearerInformationCSD;
import com.nokia.m2m.orb.idl.properties.ParametersPackage.WAPBearerInformationGPRS;
import com.nokia.m2m.orb.idl.properties.ParametersPackage.WAPBearerInformationUnion;
import com.nokia.m2m.orb.idl.properties.ParametersPackage.WAPCSDBitRate;
import com.nokia.m2m.orb.idl.properties.ParametersPackage.WAPParametersAuthentication;
import com.nokia.m2m.orb.idl.properties.ParametersPackage.WAPParametersAuthenticationHelper;
import com.nokia.m2m.orb.idl.properties.ParametersPackage.WAPParametersDefaultConnectionHelper;

import com.nokia.m2m.orb.idl.properties.ParametersPackage.WAPParametersHelper;
import com.nokia.m2m.orb.idl.properties.ParametersPackage.WAPParametersHolder;
import com.nokia.m2m.orb.idl.wirelessdevice.Device;
import com.nokia.m2m.orb.idl.wirelessdevice.DeviceHelper;
import com.nokia.m2m.sdk.examples.logger.SerialPortLogger;

/**
 *  This example IMlet sets wireless bearer settings into the Nokia 12 module.
 * 
 *  - Create one GPRS wireless bearer into the first connection index
 *  - Create one CSD wireless bearer into the second connection index
 *  - Set created GPRS connection as default connection
 *  - Set HLA information
 *  - Set information needed to authenticate incoming calls
 *  - Set local IP address
 * 
 */
public class BearerSettings extends MIDlet {

	// PPP username	and password
	final String pppUserName = "username";
	final String pppPassword = "secret";
	final String gprsAccesPointName = "ap.example.com";

	// Modem pool dial number. This is needed if the CSD bearer is used
	final String csdModemPoolNumer = "";

	// Gateway address is required only if wake-up service is used and
	// it can be left empty in most cases.
	//final String gwIPAddress = "10.35.1.195";
	final String gwIPAddress = "";
	final short gwPort = 0;

	// Connection timeout. This parameter is not needed for GPRS,
	// but it is recommended to set it for CSD
	final int timeoutGprs = 0;
	final int timeoutCsd = 120;

	// Home Location Agent port address. This value is not used in
	// IIOP mode.
	// final String HLAAddress = "10.35.1.201";
	final String HLAAddress = "";
	final short HLAPort = 0;

	// Any address can be used.
	final String localIpAddress = "10.35.1.5";

	private ORB orb = null;
	private Device device = null;

	public void startApp() throws MIDletStateChangeException {

		try {

			log("Starting BearerSettings");

			// Initialize the ORB and get the Device object.
			orb = ORB.init(null, null);

			String url = "corbaloc::127.0.0.1:19740/Device";
			Object devRef = orb.string_to_object(url);
			device = DeviceHelper.narrow(devRef);

			// Set up one GPRS and one CSD bearer. Both
			// can be set in one method call but for
			// simplicity they are set in separate methods.

			// Set GPRS to slot 0.
			setWirelessBearerGPRS((byte) 0);
			// Set CSD to slot 1.
			setWirelessBearerCSD((byte) 1);

			// Set GPRS as the default connection
			// from slot 0.
			setDefaultConnection((byte) 0);

			// Set up HLA information
			setHLAInformation();

			// Before the Nokia 12 module can receive incoming
			// calls, it must have authentication information set.
			// This has an effect only if at least one 
			// CSD bearer is configured. 
			setIncomingCallAuthenticationInfo();

			// Set the local IP address used in the PPP layer.
			// Local IP must be set if a PC or mobile phone creates 
			// datacall to the Nokia 12 module. Also, remote 
			// IMlet loading using the Nokia 12 Configurator 
			// requires that an address
			// is set to the target Nokia 12 module. 
			setPPPIPAddress();

			log("All parameters set!");

		} catch (Exception e) {
			log(e.toString());
		}

	}

	/**
	 * Builds GPRS wireless beaerer into given connection index
	 * 
	 * @param connectionIndex 
	 * @throws Exception
	 */
	private void setWirelessBearerGPRS(short connectionIndex) throws Exception {

		log("Setting wireless bearer GPRS");

		//
		// Construct the WAPBearerInformation[] structure. 
		//

		// Set one wireless bearer. Multiple bearers can be set by one 
		// method invocation.
		WAPBearerInformation wapBearerInfoArray[] = new WAPBearerInformation[1];

		// Initialize the bearer array.
		for (int i = 0; i < wapBearerInfoArray.length; i++) {
			wapBearerInfoArray[i] = new WAPBearerInformation();
		}

		WAPBearerInformationUnion wapBearerInfoUnionGprs = new WAPBearerInformationUnion();

		//
		// Construct the GPRS bearer structure.
		//
		WAPBearerInformationGPRS gprsInfo =
			new WAPBearerInformationGPRS(timeoutGprs, gwIPAddress, gprsAccesPointName, pppUserName, pppPassword);
		wapBearerInfoUnionGprs.bearerInfoGPRS(gprsInfo);

		wapBearerInfoArray[0].WAPbearerInformationUnion = wapBearerInfoUnionGprs;

		// Set port numbers and the connection index
		// zero-based indexing is used here.
		for (int i = 0; i < wapBearerInfoArray.length; i++) {
			// This value is needed only for Nokia 30 compatibility.
			// The value must be set to zero. 
			wapBearerInfoArray[i].ETPort = (short) (0);
			wapBearerInfoArray[i].GWPort = gwPort;
			wapBearerInfoArray[i].connectionIndex = connectionIndex;
		}

		WAPParametersHolder wapParameters = new WAPParametersHolder(wapBearerInfoArray);

		// Insert the created structure into CORBA Any.
		org.omg.CORBA.Any wapParamAny = orb.create_any();
		WAPParametersHelper.insert(wapParamAny, wapBearerInfoArray);

		// Set the parameters.
		device.setParam("WAPParameters", wapParamAny);

		log("Wireless bearer GPRS OK!");

	}

	/**
	 * Builds CSD wireless bearer into given connection index
	 * 
	 * @param connectionIndex 
	 * @throws Exception
	 */
	private void setWirelessBearerCSD(short connectionIndex) throws Exception {

		log("Setting wireless bearer CSD");

		//
		// Construct the WAPBearerInformation[] structure. 
		//

		// Set one wireless bearer. Multiple bearers can be set by one 
		// method invoke.
		WAPBearerInformation wapBearerInfoArray[] = new WAPBearerInformation[1];

		// Init array
		for (int i = 0; i < wapBearerInfoArray.length; i++) {
			wapBearerInfoArray[i] = new WAPBearerInformation();
		}

		WAPBearerInformationUnion wapBearerInfoUnionCsd = new WAPBearerInformationUnion();

		//
		// Create the CSD bearer.
		//
		WAPCSDBitRate bitRate = WAPCSDBitRate.NONTRANSP_9600;

		WAPBearerInformationCSD csdInfo =
			new WAPBearerInformationCSD(timeoutCsd, bitRate, csdModemPoolNumer, gwIPAddress, pppUserName, pppPassword);

		wapBearerInfoUnionCsd.bearerInfoCSD(csdInfo);

		wapBearerInfoArray[0].WAPbearerInformationUnion = wapBearerInfoUnionCsd;

		// Set the port numbers and the connection index.
		// Zero-based indexing is used here.
		for (int i = 0; i < wapBearerInfoArray.length; i++) {
			// This value is needed only for Nokia 30 compatibility.
			// The value must be set to zero. 
			wapBearerInfoArray[i].ETPort = (short) (0);
			wapBearerInfoArray[i].GWPort = gwPort;
			wapBearerInfoArray[i].connectionIndex = connectionIndex;
		}

		WAPParametersHolder wapParameters = new WAPParametersHolder(wapBearerInfoArray);

		// Insert the created structure into CORBA Any.
		org.omg.CORBA.Any wapParamAny = orb.create_any();
		WAPParametersHelper.insert(wapParamAny, wapBearerInfoArray);

		// Set the parameters.
		device.setParam("WAPParameters", wapParamAny);

		log("Wireless bearer CSD OK!");

	}

	/**
	 * Set given connection as 
	 * Nokia 12 default connection.
	 * 
	 * @param connectionIndex
	 * @throws Exception
	 */
	private void setDefaultConnection(byte connectionIndex) throws Exception {

		log("Setting default connection " + connectionIndex);

		Any any = orb.create_any();
		WAPParametersDefaultConnectionHelper.insert(any, connectionIndex);
		device.setParam("WAPParametersDefaultConnection", any);

		log("Default connection set");

	}

	/**
	 * Set Home Location Agent (HLA) information. HLA information is 
	 * needed when Nokia M2M platform is used.
	 * 
	 * @throws Exception
	 */
	private void setHLAInformation() throws Exception {
		log("Setting HLA information");

		Any any = orb.create_any();
		HLAInformation hlaInfo = new HLAInformation(HLAAddress, HLAPort);
		HLAInformationHelper.insert(any, hlaInfo);
		device.setParam("HLAInformation", any);

		log("HLA information OK");
	}

	/**
	 * Set authentication information needed to
	 * receive an incoming call.
	 * 
	 */
	private void setIncomingCallAuthenticationInfo() throws Exception {
		log("Setting incoming call authentication information");

		// Prepare structure
		Any any = orb.create_any();

		// No CSD number authentication. Enable CHAP authentication
		WAPParametersAuthentication authInfo = new WAPParametersAuthentication(false, true, pppUserName, pppPassword);

		WAPParametersAuthenticationHelper.insert(any, authInfo);

		device.setParam("WAPParametersAuthentication", any);

		log("Incoming call authentication information OK");

	}

	/**
	 * Set local IP address used in PPP layer 
	 * for incoming datacalls.
	 * 
	 * @throws Exception
	 */
	private void setPPPIPAddress() throws Exception {
		log("Setting PPP IP address");

		Any any = orb.create_any();
		PPPNegotiationIPAddressHelper.insert(any, localIpAddress);
		device.setParam("PPPNegotiationIPAddress", any);

		log("PPP IP address OK");

	}

	/**
	 * Write log message to serial port
	 * @param msg Message to write
	 */
	private void log(String msg) {
		SerialPortLogger.getInstance().write(msg);
	}

	public void pauseApp() {
	}

	public void destroyApp(boolean arg0) throws MIDletStateChangeException {
	}

}
