package com.nokia.m2m.sdk.examples.device;

import org.omg.CORBA.Any;
import org.omg.CORBA.AnyHolder;
import org.omg.CORBA.ORB;
import org.omg.CORBA.Object;

import com.nokia.m2m.orb.idl.properties.ParametersPackage.ModuleORBMode;
import com.nokia.m2m.orb.idl.properties.ParametersPackage.ModuleORBModeHelper;
import com.nokia.m2m.orb.idl.properties.ParametersPackage.RS232Mode;
import com.nokia.m2m.orb.idl.properties.ParametersPackage.RS232ModeInformation;
import com.nokia.m2m.orb.idl.properties.ParametersPackage.RS232PortModeListHelper;
import com.nokia.m2m.orb.idl.wirelessdevice.DeviceHelper;
import com.nokia.m2m.sdk.examples.logger.SerialPortLogger;

/**
 * This class contains some helper methods for Nokia 12 module configuration.
 *
 * 
 */
public class ModuleUtils {

	/**
	 * Method checks if module ORB is in IIOP mode. If the module ORB is not
	 * in IIOP mode, the method sets the module ORB to IIOP mode and resets module.
	 * 
	 * @param orb Initialized ORB
	 * @return True if module is going to reset
	 */
	public static boolean setToIIOPMode(ORB orb) {

		try {

			// Ask mode	
			log("Checking if ModuleORB is in IIOP mode...");

			Object deviceRef = orb.string_to_object("corbaloc::127.0.0.1:19740/Device");
			com.nokia.m2m.orb.idl.wirelessdevice.Device device = DeviceHelper.narrow(deviceRef);

			AnyHolder anyh = new AnyHolder();
			device.getParam("ModuleORBMode", anyh);
			ModuleORBMode oldMode = ModuleORBModeHelper.extract(anyh.value);
			if (oldMode.value() != ModuleORBMode._IIOP) {

				log("ModuleORB is not in IIOP mode. Setting ModuleORB to IIOP mode");
				Any any = orb.create_any();
				ModuleORBMode mode = ModuleORBMode.IIOP;
				ModuleORBModeHelper.insert(any, mode);
				device.setParam("ModuleORBMode", any);
				log("ModuleORB set to IIOP mode, resetting module...");

				device.reset();

				return true;

			} else {
				log("ModuleORB is already in IIOP mode");
			}

		} catch (Exception ignored) {

		}

		return false;
	}

	private static void log(String msg) {
		SerialPortLogger.getInstance().write(msg);
	}

	/**
	 * Set up serial port configuration. Reset module if configuration
	 * change was needed.
	 * 
	 * Port Settings:
	 * port1 = GPS
	 * port2 = System Protocol for Nokia 12 Configurator 
	 * port3 = Java serial port
	 * 
	 * @param orb Initialized ORB
	 * @return True if module is going to reset
	 */
	public static boolean setUpSerialPorts(ORB orb) {

		try {

			// Ask mode	
			log("Checking serial ports ...");

			Object deviceRef = orb.string_to_object("corbaloc::127.0.0.1:19740/Device");
			com.nokia.m2m.orb.idl.wirelessdevice.Device device = DeviceHelper.narrow(deviceRef);

			AnyHolder anyHolder = new AnyHolder();
			device.getParam("RS232PortModeList", anyHolder);

			RS232ModeInformation[] currentModes = RS232PortModeListHelper.extract(anyHolder.value);

			// List current modes to log
			log("Portmodes: ");
			for (int i = 0; i < currentModes.length; i++) {
				log("port " + currentModes[i].port + " mode " + currentModes[i].mode.value() + "\r\n");
			}

			// Check each port against target setup
			boolean changes = false;
			for (int i = 0; i < currentModes.length; i++) {

				// Check only port 1 and port 3 because only the settings
				// of those ports can be modified
				if (currentModes[i].port == 1 && currentModes[i].mode != RS232Mode.GPS) {
					log("Need to change serial port " + currentModes[i].port + " mode");
					changes = true;
				} else if (currentModes[i].port == 3 && currentModes[i].mode != RS232Mode.JAVA) {
					log("Need to change changing serial port " + currentModes[i].port + " mode");
					changes = true;
				}

			}

			// Only port 1 and port 3 settigns can be modified. Attempt to modify port 2
			// will throw an exception.

			// Write new settings only if changes are needed.
			if (changes) {

				log("Change settings");

				RS232ModeInformation port1Mode = new RS232ModeInformation(1, RS232Mode.GPS);
				RS232ModeInformation port3Mode = new RS232ModeInformation(3, RS232Mode.JAVA);

				RS232ModeInformation[] newInformation = new RS232ModeInformation[] { port1Mode, port3Mode };

				Any any = orb.create_any();
				RS232PortModeListHelper.insert(any, newInformation);
				device.setParam("RS232PortModeList", any);

				device.reset();
				return true;
			} else {
				log("Serial port configuration is OK");
			}

		} catch (Exception e) {
			log("setUpSerialPorts():" + e.toString());
		}

		return false;
	}

}
