package com.nokia.m2m.sdk.examples.logger;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.util.Vector;

import javax.microedition.rms.RecordEnumeration;
import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordStoreException;
import javax.microedition.rms.RecordStoreFullException;
import javax.microedition.rms.RecordStoreNotFoundException;

import org.omg.CORBA.ORB;
import org.omg.CORBA.Object;
import org.omg.CORBA.Policy;
import org.omg.PortableServer.IdAssignmentPolicyValue;
import org.omg.PortableServer.IdUniquenessPolicyValue;
import org.omg.PortableServer.LifespanPolicyValue;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;

import com.nokia.m2m.sdk.examples.logger.idl.logger.LogException;
import com.nokia.m2m.sdk.examples.logger.idl.logger.LogMessage;
import com.nokia.m2m.sdk.examples.logger.idl.logger.LogPOA;

/**
 * RMSLogger writes log messages to record store files. RMSLogger Provides
 * CORBA API to read log messages messages remotely.
 * 
 */
public class RMSLogger extends LogPOA implements Log {

	private static RMSLogger instance;

	// Maximum size of one log message
	private static final int MAX_MSG_LEN = 1000;
	// If file size is too big reading will be really slow. 
	private static final int MAX_FILE_SIZE = 5000;

	private static final String FILE_NAME_START = "RmsLogFile-";

	// 5 files total, each can be 32KB size 
	// RMSLogger overrides oldest one when all files are full.
	private static final String[] FILES =
		new String[] {
			FILE_NAME_START + "0",
			FILE_NAME_START + "1",
			FILE_NAME_START + "2",
			FILE_NAME_START + "3",
			FILE_NAME_START + "4",
			};

	private ORB orb;

	// File currently in use
	private int currentIndex = 0;

	// Currently open file
	private RecordStore currentFile = null;

	// Changed to false after first file is opened
	private boolean firstSeek = true;

	private RMSLogger(ORB orb) {
		this.orb = orb;
		initServant();
	}

	/**
	 * 
	 * @param orb ORB used when activating servant
	 * @return
	 */
	public static synchronized Log getInstance(ORB orb) {
		if (instance == null) {
			instance = new RMSLogger(orb);
		}
		return instance;
	}

	/**
	 * Writes a log message to RecordStore
	 * @param msg Message to write
	 */
	public void write(String msg) {

		// No nulls
		if (msg == null || msg.length() > MAX_MSG_LEN) {
			// discard
			return;
		}

		try {

			// prepare message
			ByteArrayOutputStream out = new ByteArrayOutputStream();
			DataOutputStream dout = new DataOutputStream(out);

			// timestamo
			dout.writeLong(System.currentTimeMillis());
			// message
			dout.writeUTF(msg);
			dout.flush();

			byte[] data = out.toByteArray();

			synchronized (this) {
				if (currentFile == null) {

					try {

						currentFile = nextFile();

					} catch (Exception e) {
						// can't continue from this
						logSerial("nextFile() failed " + e.toString());
						return;
					}

				}

				if (currentFile.getSize() + data.length > MAX_FILE_SIZE) {
					currentFile = nextFile();
				}

				try {
					currentFile.addRecord(data, 0, data.length);
				} catch (RecordStoreFullException e) {
					// Move to next file
					try {
						currentFile = nextFile();
						currentFile.addRecord(data, 0, data.length);
					} catch (Exception ignored) {
						// Nothing to do
						logSerial("write():" + ignored.toString());
						return;
					}

				}

			}

		} catch (Exception ignored) {

		}

	}

	/**
	 * Close RMSLog. Close currently open file
	 */
	public void close() {

		synchronized (this) {
			try {

				if (currentFile != null) {
					currentFile.closeRecordStore();

				}

			} catch (Exception ignored) {
			}

			currentFile = null;
		}

	}

	/**
	 * Open next file. If all files are full, delete
	 * oldest one
	 * @return
	 */
	private RecordStore nextFile() throws RecordStoreException {

		RecordStore rs;
		// Try to open each RecordStore by name. If the store opens,
		// ensure that there is still room left and return the file

		logSerial("nextFile()");

		if (firstSeek) {
			firstSeek = false;

			logSerial("first seek");

			// This is the first time when this method is called.
			// Check all RecordStore files
			for (int i = 0; i < FILES.length; i++) {

				try {

					// Check if file is empty
					rs = RecordStore.openRecordStore(FILES[i], false);
					// Actual limit for file is 32KB but because RMSLogger logic
					// is based to the fact that next log write should succeed
					// we must make sure that there is enough room for at least
					// one write.

					// Check that file is empty
					if (rs.getSize() < 500) {
						// This file is not full
						currentIndex = i;
						return rs;
					}

				} catch (RecordStoreNotFoundException rnfe) {
					// Not found, this slot is free
					currentIndex = i;
					return RecordStore.openRecordStore(FILES[currentIndex], true);
				} catch (RecordStoreException re) {
					// Store is broken, delete it
					try {
						RecordStore.deleteRecordStore(FILES[i]);
					} catch (Exception ignored) {
					}
					// Use the file
					currentIndex = i;
					return RecordStore.openRecordStore(FILES[currentIndex], true);

				}

			}
			//
			// Execution falls here if all files are used and full.
			// Start from the first file.
			//
			currentIndex = 0;

			try {
				RecordStore.deleteRecordStore(FILES[currentIndex]);
			} catch (Exception e) {
			}

			return RecordStore.openRecordStore(FILES[currentIndex], true);

		} else {

			// This is not the first time this 
			// method is called. Simply move to the next file.

			// Close the old file
			if (currentFile != null) {
				try {
					currentFile.closeRecordStore();
				} catch (Exception ignored) {
				}
			}

			// Move to the next file
			if (currentIndex >= FILES.length - 1) {
				// back to file 0
				currentIndex = 0;
			} else {
				currentIndex++;
			}

			// Try to delete the existing file
			try {
				RecordStore.deleteRecordStore(FILES[currentIndex]);
			} catch (Exception ignored) {
			}
			return RecordStore.openRecordStore(FILES[currentIndex], true);

		}

	}

	/**
	 * Initialize servant. After this log messages can be read
	 * from remote applications
	 *
	 */
	private void initServant() {

		try {

			Object rpRef = orb.resolve_initial_references("RootPOA");
			POA rootPoa = POAHelper.narrow(rpRef);

			Policy[] policies =
				new Policy[] {
					rootPoa.create_id_assignment_policy(IdAssignmentPolicyValue.USER_ID),
					rootPoa.create_id_uniqueness_policy(IdUniquenessPolicyValue.UNIQUE_ID),
					rootPoa.create_lifespan_policy(LifespanPolicyValue.PERSISTENT)};

			POA loggerPoa = rootPoa.create_POA("logger", rootPoa.the_POAManager(), policies);

			loggerPoa.activate_object_with_id("RMSLogger".getBytes(), this);

			// object key is now "logger/RMSLogger"

			// Make sure that the POAmanager is active. 
			// POAManager can be activated multiple times.
			rootPoa.the_POAManager().activate();

		} catch (Exception e) {

			logSerial(e.toString());

		}

		logSerial("initServant done");

	}

	/**
	 * List log files
	 * 
	 * StringList listFiles() raises (LogException);
	 */
	public String[] listFiles() throws LogException {

		try {

			java.util.Vector tmpNames = new Vector();
			String[] rsFiles = RecordStore.listRecordStores();
			if (rsFiles != null) {

				// Seek all file names starting with 
				// FILE_NAME_START

				for (int i = 0; i < rsFiles.length; i++) {
					if (rsFiles[i].startsWith(FILE_NAME_START)) {
						tmpNames.addElement(rsFiles[i]);
					}
				}

				// Return data
				String[] ret = new String[tmpNames.size()];
				for (int i = 0; i < tmpNames.size(); i++) {
					ret[i] = (String) tmpNames.elementAt(i);
				}

				return ret;

			} else {
				// Return an empty array. Never return null in CORBA calls!
				return new String[] {
				};
			}

		} catch (Exception e) {
			throw new LogException(e.toString());
		}
	}

	/**
	 * Delete record store
	 * 
	 * void deleteFile(in string file) raises (LogException);
	 */
	public void deleteFile(String file) throws LogException {

		synchronized (this) {

			// Don't delete currently open file
			if (FILES[currentIndex].equals(file)) {
				throw new LogException("File " + FILES[currentIndex] + " is open");
			}
			try {

				RecordStore.deleteRecordStore(file);
			} catch (Exception e) {
				throw new LogException("Failed to delete file " + FILES[currentIndex] + " " + e.toString());
			}

		}

	}

	/**
	 * Read messages from RecordStore and return to caller. 
	 * This is not the fastest possible way to process data so limitation
	 * for file size is needed.
	 *				
	 * 
	 * MessageList getMessages(in string fileName) raises (LogException);
	 */
	public LogMessage[] getMessages(String fileName) throws LogException {

		try {

			if (!fileName.startsWith(FILE_NAME_START)) {
				throw new LogException("Filename must start " + FILE_NAME_START);
			}

			RecordEnumeration enum = null;
			int numOfRecords = 0;

			// Get exlusive lock to records.
			synchronized (this) {
				RecordStore rs;

				if (FILES[currentIndex].equals(fileName) && currentFile != null) {
					rs = currentFile;
				} else {

					// May throw RecordstoreException if file is broken.
					// Exception will be handled below.
					rs = RecordStore.openRecordStore(fileName, false);

				}

				// Read messages
				numOfRecords = rs.getNumRecords();

				// Get snapshot from current situation
				enum = rs.enumerateRecords(null, null, false);

				byte[] record;
				LogMessage[] ret = new LogMessage[numOfRecords];
				LogMessage logMsg;
				int index = 0;
				while (enum.hasNextElement()) {

					record = enum.nextRecord();

					ByteArrayInputStream in = new ByteArrayInputStream(record);
					DataInputStream din = new DataInputStream(in);

					logMsg = new LogMessage(din.readLong(), din.readUTF());
					ret[index++] = logMsg;

				}

				return ret;

			} // synchronized

		} catch (Exception e) {
			// Convert Java exception to exception defined in IDL
			throw new LogException(e.toString());
		}

	}

	private void logSerial(String msg) {
		// Serial port logging is used in development time to ensure that
		// the RMSLogger works correctly. 
		// Comment this line out if the serial port is needed by the IMlet.
		SerialPortLogger.getInstance().write(msg);

	}

}
