package com.nokia.m2m.sdk.examples.socket;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import javax.microedition.io.Connector;
import javax.microedition.io.StreamConnection;
import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;

/**
 * 
 * IMlet works as bridge between Serial port and server application. IMlet reads data
 * from serial port 3 and forwards data to target server using TCP connection.
 * 
 * This is nonblocking version. The IMlet checks if there is data available in the serial
 * port or socket before trying to read data.
 * 
 */
public class SerialToTCPBridge extends MIDlet {

	// TCP server to connect. All data read from serial port
	// will be sent to this server. Change address.
	private final String TCP_TARGET_ADDRESS = "10.35.1.195";
	private final int TCP_TARGET_PORT = 9001;

	// IMlet uses serial port 3
	private final int SERIAL_PORT_SPEED = 115200;

	private static final int BUFFER_SIZE = 4096;

	private InputStream serialInStream = null;
	private OutputStream serialOutStream = null;

	/**
	 * 
	 * 
	 */
	public void startApp() throws MIDletStateChangeException {

		StreamConnection socketConn = null;
		StreamConnection serialConn = null;

		InputStream socketInStream = null;
		OutputStream socketOutStream = null;

		int dataAvailable = 0;
		boolean dataTransmitted = false;

		try {

			// Open serial port

			serialConn =
				(StreamConnection) Connector.open("comm:3;baudrate=" + SERIAL_PORT_SPEED, Connector.READ_WRITE);

			// set up streams
			serialInStream = serialConn.openInputStream();
			serialOutStream = serialConn.openOutputStream();

			log("Serial port open");

		} catch (Exception e) {
			// can not continue
			log(e.toString());
			return;
		}

		byte[] buff = new byte[BUFFER_SIZE];
		int length;

		// Keep reading serial port. Send data to server every
		// time when read request returns with data. 
		while (true) {

			// This is the reconnect loop
			log("Going to reconnect loop");

			try {

				log("Going to read loop");
				while (true) {

					try {

						//
						// Check if there is data in serialport buffer
						//
						//log("Calling serial port available");
						for (dataAvailable = serialInStream.available();
							dataAvailable > 0;
							dataAvailable = serialInStream.available()) {

							if (dataAvailable > BUFFER_SIZE) {
								dataAvailable = BUFFER_SIZE;
							}

							// This won't block 
							length = serialInStream.read(buff, 0, dataAvailable);

							if (length > 0) {

								log("Received " + length + " bytes from serial port");

								if (socketConn == null) {

									socketConn = connect();
									socketOutStream = socketConn.openOutputStream();
									socketInStream = socketConn.openInputStream();

								}

								// Throws IOException if network link has been dropped
								dataTransmitted = true;
								socketOutStream.write(buff, 0, length);
								socketOutStream.flush();

								log("Sent to TCP, " + length + " bytes");

							} else {
								// Should not happen with serial port 
								log("read() returned 0");
							}

						} // for data available in serial port

						//
						// Read data from socket buffer and write to serial port 
						//
						if (socketInStream != null) {

							// Available returns 0 as long as there is no data in socket
							//log("Calling socket available");
							for (dataAvailable = socketInStream.available();
								dataAvailable > 0;
								dataAvailable = socketInStream.available()) {

								if (dataAvailable > BUFFER_SIZE) {
									dataAvailable = BUFFER_SIZE;
								}

								// Read data from socket and write to serial port
								length = socketInStream.read(buff, 0, dataAvailable);

								if (length > 0) {
									log("Received " + length + " bytes from socket");
									dataTransmitted = true;
									serialOutStream.write(buff, 0, length);
									serialOutStream.flush();
									log("Data send to serial port");
								} else {
									// Socket closed by remote peer
									throw new IOException("Socket closed by remote peer");
								}

							} // for data available in socket

						}

						// One loop completed, wait a moment before going to next round
						if (!dataTransmitted) {
							Thread.sleep(500);
						}

						// If watchdog is used, this is a good place for
						// watchdog timer update

						// Catch exceptions inside read loop
					} catch (Exception e) {

						log("Exception " + e.toString());

						if (socketOutStream != null) {
							try {
								socketOutStream.close();
							} catch (Exception ignored) {
							}

							socketOutStream = null;
						}

						if (socketInStream != null) {
							try {
								socketInStream.close();
							} catch (Exception ignored) {
							}
							socketInStream = null;
						}

						if (socketConn != null) {
							try {

								socketConn.close();
							} catch (Exception ignored) {
							}
							socketConn = null;
						}

						try {
							// wait 5 secs between connection attempts
							Thread.sleep(5000);
						} catch (InterruptedException ignore) {
						}

					}

				} // read loop

			} catch (Exception e) {
				log(e.toString());
			}

		} // re-initialization loop

	}

	/**
	 * Connect to server
	 * 
	 * @return StreamConnection
	 * @throws IOException
	 */
	private StreamConnection connect() throws IOException {

		final String url = "socket://" + TCP_TARGET_ADDRESS + ":" + TCP_TARGET_PORT;
		StreamConnection conn = null;

		log("Connecting server: " + url);
		conn = (StreamConnection) Connector.open(url);
		log("Connected");
		return conn;

	}

	/**
	 * Write log message to serial port
	 * 
	 * @param msg
	 */
	private void log(String msg) {

		// Comment off these rows to remove logging to serial port

		msg = "[" + msg + "]";

		// can't log to serial port
		if (serialOutStream != null) {
			try {
				serialOutStream.write(msg.getBytes());
			} catch (Exception ignored) {
			}
		}

	}

	public void pauseApp() {
		// Not needed
	}

	public void destroyApp(boolean arg0) throws MIDletStateChangeException {
		// Not needed
	}

}
