/**
 * PositionDB
 *
 * @author t533807
 * @version 1.0
 */
package com.telenor.agps;

import org.apache.log4j.Logger;

import java.sql.*;
import java.util.Vector;
import java.util.prefs.Preferences;

public class DBService {
	private static DBService ourInstance;
	private Connection conn;
	private final static Logger logger = Logger.getLogger(DBService.class);
	private final static boolean DEBUG = false;

	public synchronized static DBService getInstance() {
		if (ourInstance == null) {
			ourInstance = new DBService();
		}
		return ourInstance;
	}

	private DBService() {
		try {
			connect();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private void connect() throws Exception {
		Preferences prefs = Preferences.userNodeForPackage(this.getClass());
		String db_url = prefs.get("DB_URL", "jdbc:mysql://10.1.2.4/agps");
		String username = prefs.get("DB_USER", "root");
		String password = prefs.get("DB_PASSWORD", "");
		Class.forName("com.mysql.jdbc.Driver").newInstance();
		conn = DriverManager.getConnection(db_url, username, password);
	}

	public void shutdown() {
		try {
			conn.close();   // if there are no other open connection
			// db writes out to files and shuts down
			// this happens anyway at garbage collection
			// when program ends
		} catch (SQLException e) {
		}
	}

	public synchronized Vector getAlmanac() {
		String sel_expr = "SELECT * " +
				  "FROM almanac ";

		if (!checkConnection())
			return null;

		Vector almanacs = new Vector(32);
		try {
			// Get almanac data from db
			if (DEBUG) logger.debug(sel_expr);
			PreparedStatement pst = conn.prepareStatement(sel_expr);
			ResultSet result = execQuery(pst);
			while (result.next()) {
				Almanac almanac = new Almanac();
				almanac.prn = result.getInt("prn");
				almanac.health = result.getInt("health");
				almanac.e = result.getDouble("e");
				almanac.toa = result.getLong("toa");
				almanac.i0 = result.getDouble("i0");
				almanac.omegadot = result.getDouble("omegadot");
				almanac.roota = result.getDouble("roota");
				almanac.omega0 = result.getDouble("omega0");
				almanac.omega = result.getDouble("omega");
				almanac.m0 = result.getDouble("m0");
				almanac.af0 = result.getFloat("af0");
				almanac.af1 = result.getFloat("af1");
				almanac.wna = result.getInt("wna");
				almanac.wn = result.getInt("wn");
				almanac.tow = result.getLong("tow");
				almanacs.add(almanac);
			}
			pst.close();
			return almanacs;
		} catch (SQLException e) {
			e.printStackTrace();
		}

		return null;
	}

	public synchronized void setAlmanac(Almanac almanac) {
		String del_expr = "DELETE FROM almanac WHERE prn = ?";
		String ins_expr = "INSERT INTO almanac VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";

		if (!checkConnection())
			return;

		try {
			// Delete almanac data from db
			if (DEBUG) logger.debug(del_expr);
			PreparedStatement pst = conn.prepareStatement(del_expr);
			pst.setInt(1, almanac.prn);
			pst.executeUpdate();
			pst.close();
		} catch (SQLException e) {
			e.printStackTrace();
		}

		try {
			// Save almanac data in db
			if (DEBUG) logger.debug(ins_expr);
			PreparedStatement pst = conn.prepareStatement(ins_expr);
			pst.setInt(1, almanac.prn);
			pst.setInt(2, almanac.health);
			pst.setDouble(3, almanac.e);
			pst.setLong(4, almanac.toa);
			pst.setDouble(5, almanac.i0);
			pst.setDouble(6, almanac.omegadot);
			pst.setDouble(7, almanac.roota);
			pst.setDouble(8, almanac.omega0);
			pst.setDouble(9, almanac.omega);
			pst.setDouble(10, almanac.m0);
			pst.setDouble(11, almanac.af0);
			pst.setDouble(12, almanac.af1);
			pst.setInt(13, almanac.wna);
			pst.setInt(14, almanac.wn);
			pst.setLong(15, almanac.tow);

			execUpdate(pst);
			pst.close();
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return;
	}

	public synchronized Vector getEphemeris() {
		String sel_expr = "SELECT * " +
				  "FROM ephemeris ";

		if (!checkConnection())
			return null;

		Vector ephemerises = new Vector(32);
		try {
			// Get almanac data from db
			if (DEBUG) logger.debug(sel_expr);
			PreparedStatement pst = conn.prepareStatement(sel_expr);
			ResultSet result = execQuery(pst);
			while (result.next()) {
				Ephemeris eph = new Ephemeris();
				eph.prn = result.getInt("prn");
				eph.wn = result.getInt("wn");
				eph.tow = result.getLong("tow");
				eph.tgd = result.getFloat("tgd");
				eph.aodc = result.getLong("aodc");
				eph.toc = result.getLong("toc");
				eph.af2 = result.getFloat("af2");
				eph.af1 = result.getFloat("af1");
				eph.af0 = result.getFloat("af0");
				eph.aode = result.getLong("aode");
				eph.deltan = result.getFloat("deltan");
				eph.m0 = result.getDouble("m0");
				eph.e = result.getDouble("e");
				eph.roota = result.getDouble("roota");
				eph.toe = result.getLong("toe");
				eph.cic = result.getFloat("cic");
				eph.crc = result.getFloat("crc");
				eph.cis = result.getFloat("cis");
				eph.crs = result.getFloat("crs");
				eph.cuc = result.getFloat("cuc");
				eph.cus = result.getFloat("cus");
				eph.omega0 = result.getDouble("omega0");
				eph.omega = result.getDouble("omega");
				eph.i0 = result.getDouble("i0");
				eph.omegadot = result.getFloat("omegadot");
				eph.idot = result.getFloat("idot");
				eph.accuracy = result.getInt("accuracy");
				eph.health = result.getInt("health");
				eph.fit = result.getInt("fit");
				ephemerises.add(eph);
			}
			pst.close();
			return ephemerises;
		} catch (SQLException e) {
			e.printStackTrace();
		}

		return null;
	}

	public synchronized void setEphemeris(Ephemeris eph) {
		String del_expr = "DELETE FROM ephemeris WHERE prn = ?";
		String ins_expr = "INSERT INTO ephemeris VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";

		if (!checkConnection())
			return;

		try {
			// Delete almanac data from db
			if (DEBUG) logger.debug(del_expr);
			PreparedStatement pst = conn.prepareStatement(del_expr);
			pst.setInt(1, eph.prn);
			execUpdate(pst);
			pst.close();
		} catch (SQLException e) {
			e.printStackTrace();
		}

		try {
			// Save almanac data in db
			if (DEBUG) logger.debug(ins_expr);
			PreparedStatement pst = conn.prepareStatement(ins_expr);
			pst.setInt(1, eph.prn);
			pst.setInt(2, eph.wn);
			pst.setLong(3, eph.tow);
			pst.setFloat(4, eph.tgd);
			pst.setLong(5, eph.aodc);
			pst.setLong(6, eph.toc);
			pst.setFloat(7, eph.af2);
			pst.setFloat(8, eph.af1);
			pst.setFloat(9, eph.af0);
			pst.setLong(10, eph.aode);
			pst.setFloat(11, eph.deltan);
			pst.setDouble(12, eph.m0);
			pst.setDouble(13, eph.e);
			pst.setDouble(14, eph.roota);
			pst.setLong(15, eph.toe);
			pst.setFloat(16, eph.cic);
			pst.setFloat(17, eph.crc);
			pst.setFloat(18, eph.cis);
			pst.setFloat(19, eph.crs);
			pst.setFloat(20, eph.cuc);
			pst.setFloat(21, eph.cus);
			pst.setDouble(22, eph.omega0);
			pst.setDouble(23, eph.omega);
			pst.setDouble(24, eph.i0);
			pst.setFloat(25, eph.omegadot);
			pst.setFloat(26, eph.idot);
			pst.setInt(27, eph.accuracy);
			pst.setInt(28, eph.health);
			pst.setInt(29, eph.fit);

			execUpdate(pst);
			pst.close();
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return;
	}

	private ResultSet execQuery(PreparedStatement pst) {
		int retry = 2;
		ResultSet res = null;
		while (retry > 0) {
			try {
				res = pst.executeQuery();
				return res;
			} catch (SQLException e) {
				String sqlState = e.getSQLState();
				if ("08S01".equals(sqlState) || "41000".equals(sqlState)) {
					// Reconnect
					try {
						connect();
					} catch (Exception e1) {
						e1.printStackTrace();
					}
					retry--;
				} else {
					retry = 0;
				}
			}
		}
		return res;
	}

	private void execUpdate(PreparedStatement pst) {
		int retry = 2;
		while (retry > 0) {
			try {
				pst.executeUpdate();
				return;
			} catch (SQLException e) {
				String sqlState = e.getSQLState();
				if ("08S01".equals(sqlState) || "41000".equals(sqlState)) {
					// Reconnect
					try {
						connect();
					} catch (Exception e1) {
						e1.printStackTrace();
					}
					retry--;
				} else {
					retry = 0;
				}
			}
		}
	}

	private boolean checkConnection() {
		try {
			if (conn.isClosed())
				connect();
		} catch (Exception e) {
			// Retry once
			try {
				shutdown();
				connect();
			} catch (Exception e2) {
				logger.error("Unable to connect to database: " + e2.getMessage());
				return false;
			}
		}
		return true;
	}

	public void execute(String sel_expr) {
		if (!checkConnection())
			return;

		try {
			Statement stmt = conn.createStatement();
			stmt.executeUpdate(sel_expr);
			stmt.close();
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}

	public Connection getConnection() {
		if (!checkConnection())
			return null;
		return conn;
	}
}

